from nonebot_plugin_imageutils import BuildImage

from .const import RES_PATH

RES_CALENDER_BANNER = BuildImage.open(RES_PATH / "calender_banner.png")
# RES_SCHALE_BG = BuildImage.open(RES_PATH / "schale_bg.jpg")
RES_GRADIENT_BG = BuildImage.open(RES_PATH / "gradient.png")
RES_GACHA_BG = BuildImage.open(RES_PATH / "gacha_bg.png")
RES_GACHA_CARD_BG = BuildImage.open(RES_PATH / "gacha_card_bg.png")
RES_GACHA_CARD_MASK = BuildImage.open(RES_PATH / "gacha_card_mask.png").convert("RGBA")
RES_GACHA_NEW = BuildImage.open(RES_PATH / "gacha_new.png")
RES_GACHA_PICKUP = BuildImage.open(RES_PATH / "gacha_pickup.png")
RES_GACHA_STAR = BuildImage.open(RES_PATH / "gacha_star.png")
RES_GACHA_STU_ERR = BuildImage.open(RES_PATH / "gacha_stu_err.png")



import asyncio
import json
import random
from dataclasses import dataclass
from io import BytesIO
from typing import Dict, Iterable, List, Optional, Tuple, TypedDict

import aiofiles
from nonebot import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot_plugin_imageutils import BuildImage

from .const import DATA_PATH
from .data_schaledb import (
    schale_get,
    schale_get_stu_dict,
)
from .resource import (
    RES_GACHA_BG,
    RES_GACHA_CARD_BG,
    RES_GACHA_CARD_MASK,
    RES_GACHA_NEW,
    RES_GACHA_PICKUP,
    RES_GACHA_STAR,
    RES_GACHA_STU_ERR,
)
from .util import split_list

GACHA_DATA_PATH = DATA_PATH / "gacha.json"
if not GACHA_DATA_PATH.exists():
    GACHA_DATA_PATH.write_text("{}")


class GachaData(TypedDict):
    collected: List[int]
    total_count: int


DEFAULT_GACHA_DATA: GachaData = {"collected": [], "total_count": 0}


@dataclass()
class GachaStudent:
    id: int
    new: bool = False
    pickup: bool = False


async def set_gacha_data(qq: str, data: GachaData):
    async with aiofiles.open(str(GACHA_DATA_PATH), "r+", encoding="u8") as f:
        j = json.loads(await f.read())
        j[qq] = data

        await f.seek(0)
        await f.truncate()

        await f.write(json.dumps(j))


async def get_gacha_data(qq: str) -> GachaData:
    async with aiofiles.open(str(GACHA_DATA_PATH), encoding="u8") as f:
        j = await f.read()

    data: Dict[str, GachaData] = json.loads(j)
    if not (user_data := data.get(qq)):
        user_data = DEFAULT_GACHA_DATA.copy()
        await set_gacha_data(qq, user_data)

    return user_data


async def gen_stu_img(students: Iterable[GachaStudent]) -> Tuple[BuildImage]:
    stu_li = await schale_get_stu_dict("Id")

    async def gen_single(stu: GachaStudent) -> BuildImage:
        bg = RES_GACHA_CARD_BG.copy()

        stu_star = 0
        try:
            stu_j = stu_li[stu.id]
            stu_star: int = stu_j["StarGrade"]
            stu_img = await schale_get(
                f"images/student/collection/{stu_j['CollectionTexture']}.webp",
                True,
            )
            stu_img = BuildImage.open(BytesIO(stu_img))
        except:
            logger.exception(f"学生数据获取失败 {stu.id}")
            stu_img = RES_GACHA_STU_ERR

        card_img = BuildImage.new("RGBA", RES_GACHA_CARD_MASK.size, (0, 0, 0, 0))
        card_img.image.paste(
            stu_img.resize(RES_GACHA_CARD_MASK.size, keep_ratio=True).image,
            mask=RES_GACHA_CARD_MASK.image,
        )

        bg = bg.paste(card_img, (26, 13), True)

        star_x_offset = int(26 + (159 - 30 * stu_star) / 2)
        star_y_offset = 198
        for i in range(stu_star):
            bg = bg.paste(RES_GACHA_STAR, (star_x_offset + i * 30, star_y_offset), True)

        font_x_offset = 45
        font_y_offset = 2

        if stu.new:
            bg = bg.paste(RES_GACHA_NEW, (font_x_offset, font_y_offset), True)
            font_x_offset -= 2
            font_y_offset += 29

        if stu.pickup:
            font_x_offset -= 4
            font_y_offset -= 4
            bg = bg.paste(RES_GACHA_PICKUP, (font_x_offset, font_y_offset), True)

        return bg

    return await asyncio.gather(*[gen_single(x) for x in students])  # noqa


async def gen_gacha_img(
    students: Iterable[GachaStudent], count: int
) -> Optional[BytesIO]:
    line_limit = 5
    stu_cards = split_list(await gen_stu_img(students), line_limit)
    if not stu_cards:
        return
    card_w, card_h = stu_cards[0][0].size

    bg = RES_GACHA_BG.copy()

    x_gap = 10
    y_gap = 80
    y_offset = int((bg.height - (len(stu_cards) * (y_gap + card_h) - y_gap)) / 2)
    for line in stu_cards:
        x_offset = int((bg.width - (len(line) * (x_gap + card_w) - x_gap)) / 2)
        for card in line:
            bg = bg.paste(card, (x_offset, y_offset), True)
            x_offset += card_w + x_gap
        y_offset += card_h + y_gap

    bg = bg.draw_text(
        (1678, 841, 1888, 885), "距上个3★UP", 30, weight="bold", fill=(36, 90, 126)
    ).draw_text(
        (1643, 885, 1890, 935), str(count), 30, weight="bold", fill=(255, 255, 255)
    )

    return bg.save("PNG")


async def gacha(qq: str, times: int, gacha_data: dict, up_pool: List[int] = None):
    # 屎山代码 别骂了别骂了
    # 如果有大佬指点指点怎么优化或者愿意发个PR就真的太感激了

    if not up_pool:
        up_pool = []

    stu_li = await schale_get_stu_dict("Id")
    up_3_li, up_2_li = [
        [x for x in up_pool if x in stu_li and stu_li[x]["StarGrade"] == y]
        for y in [3, 2]
    ]

    base_char: dict = gacha_data["base"]
    for up in up_pool:
        for li in base_char.values():  # type: List[int]
            if up in li:
                li.remove(up)

    star_3_base, star_2_base, star_1_base = [base_char[x] for x in ["3", "2", "1"]]
    star_3_chance, star_2_chance, star_1_chance = [
        x["chance"] for x in [star_3_base, star_2_base, star_1_base]
    ]

    up_3_chance = 0
    up_2_chance = 0
    if up_3_li:
        up_3_chance = gacha_data["up"]["3"]["chance"]
        star_3_chance -= up_3_chance
    if up_2_li:
        up_2_chance = gacha_data["up"]["2"]["chance"]
        star_2_chance -= up_2_chance

    gacha_data = await get_gacha_data(qq)
    gacha_result: List[GachaStudent] = []

    picked_3star_up = False
    last_count = gacha_data["total_count"]
    for i in range(1, times + 1):
        pool_and_weight = [
            (up_3_li, up_3_chance),
            (up_2_li, up_2_chance),
            (star_3_base["char"], star_3_chance),
            (star_2_base["char"], star_2_chance),
        ]
        if not i % 10 == 0:
            pool_and_weight.append((star_1_base["char"], star_1_chance))

        pool_and_weight = [x for x in pool_and_weight if x[0]]
        pool = [x[0] for x in pool_and_weight]
        weight = [x[1] for x in pool_and_weight]

        random.seed()
        char = random.choice(random.choices(pool, weights=weight, k=1)[0])

        is_3star_pickup = char in up_3_li
        is_pickup = is_3star_pickup or (char in up_2_li)
        is_new = char not in gacha_data["collected"]
        gacha_result.append(GachaStudent(id=char, pickup=is_pickup, new=is_new))

        if is_new:
            gacha_data["collected"].append(char)

        if is_3star_pickup or ((not up_pool) and char in star_3_base["char"]):
            gacha_data["total_count"] = 0
            picked_3star_up = True
        else:
            gacha_data["total_count"] += 1
            if not picked_3star_up:
                last_count += 1

    await set_gacha_data(qq, gacha_data)
    return MessageSegment.image(await gen_gacha_img(gacha_result, last_count))



from nonebot import require
from nonebot.plugin import PluginMetadata

require("nonebot_plugin_apscheduler")
require("nonebot_plugin_htmlrender")
require("nonebot_plugin_imageutils")

from .__main__ import *  # type:ignore

__version__ = "0.6.3"
__plugin_meta__ = PluginMetadata(
    name="BAWiki",
    description="碧蓝档案Wiki插件",
    usage=(
        "感谢各位sensei使用本插件！\n"
        "插件有缓存机制，每3小时自动清空一次，如果插件遇到问题可以先手动清空缓存试一下捏\n"
        "装载PicMenu插件即可查看插件详细菜单哦\n"
        "祝玩得愉快～"
    ),
    extra={
        "menu_template": "default",
        "menu_data": [
            {
                "func": "日程表",
                "trigger_method": "指令",
                "trigger_condition": "ba日程表",
                "brief_des": "查看活动日程表",
                "detail_des": (
                    "查看当前未结束的卡池、活动以及起止时间，"
                    "默认为GameKee源，可以在指令后带参数使用SchaleDB数据源\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba日程表</ft> （GameKee源）\n"
                    "- <ft color=(238,120,0)>ba日程表 schaledb</ft> （SchaleDB源，日服国际服一起发）\n"
                    "- <ft color=(238,120,0)>ba日程表 schale 国际服</ft> （SchaleDB源，国际服）"
                ),
            },
            {
                "func": "学生图鉴",
                "trigger_method": "指令",
                "trigger_condition": "ba学生图鉴",
                "brief_des": "查询学生详情（SchaleDB）",
                "detail_des": (
                    "访问对应学生SchaleDB页面并截图，支持部分学生别名\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba学生图鉴 白子</ft>\n"
                    "- <ft color=(238,120,0)>ba学生图鉴 xcw</ft>"
                ),
            },
            {
                "func": "学生Wiki",
                "trigger_method": "指令",
                "trigger_condition": "ba学生wiki",
                "brief_des": "查询学生详情（GameKee）",
                "detail_des": (
                    "访问对应学生GameKee Wiki页面并截图，支持部分学生别名\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba学生wiki 白子</ft>\n"
                    "- <ft color=(238,120,0)>ba学生wiki xcw</ft>"
                ),
            },
            {
                "func": "羁绊查询",
                "trigger_method": "指令",
                "trigger_condition": "ba羁绊",
                "brief_des": "查询学生解锁L2D需求的羁绊等级",
                "detail_des": (
                    "使用学生名称查询该学生解锁L2D看板需求的羁绊等级以及L2D预览，"
                    "或者使用羁绊等级级数查询哪些学生达到该等级时解锁L2D看板\n"
                    "使用学生名称查询时支持部分学生别名\n"
                    " \n"
                    "可以用这些指令触发：\n"
                    "- <ft color=(238,120,0)>ba羁绊</ft>\n"
                    "- <ft color=(238,120,0)>ba好感度</ft>\n"
                    "- <ft color=(238,120,0)>bal2d</ft>\n"
                    "- <ft color=(238,120,0)>balive2d</ft>\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba羁绊 xcw</ft>\n"
                    "- <ft color=(238,120,0)>ba羁绊 9</ft>"
                ),
            },
            {
                "func": "角色评价",
                "trigger_method": "指令",
                "trigger_condition": "ba角评",
                "brief_des": "查询学生角评一图流",
                "detail_des": (
                    "发送一张指定角色的评价图\n"
                    "支持部分学生别名\n"
                    "角评图作者 B站@夜猫咪喵喵猫\n"
                    " \n"
                    "可以使用 `all` / `总览` / `全部` 参数 查看全学生角评一图流\n"
                    " \n"
                    "可以用这些指令触发：\n"
                    "- <ft color=(238,120,0)>ba学生评价</ft>\n"
                    "- <ft color=(238,120,0)>ba角评</ft>\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba学生评价 白子</ft>\n"
                    "- <ft color=(238,120,0)>ba角评 xcw</ft>"
                    "- <ft color=(238,120,0)>ba角评 总览</ft>"
                ),
            },
            {
                "func": "总力战一图流",
                "trigger_method": "指令",
                "trigger_condition": "ba总力战",
                "brief_des": "查询总力战推荐配队/Boss机制",
                "detail_des": (
                    "发送当前或指定总力战Boss的配队/机制一图流攻略图\n"
                    "支持部分Boss别名\n"
                    "图片作者 B站@夜猫咪喵喵猫\n"
                    " \n"
                    "使用 <ft color=(238,120,0)>ba总力战 -h</ft> 查询指令用法\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba总力战</ft>（日服&国际服当前总力战Boss配队攻略）\n"
                    "- <ft color=(238,120,0)>ba总力战 -s j</ft>（日服当前总力战Boss配队攻略）\n"
                    "- <ft color=(238,120,0)>ba总力战 -s j -w</ft>（日服当前总力战Boss机制图）\n"
                    "- <ft color=(238,120,0)>ba总力战 寿司</ft>（Kaiten FX Mk.0 配队攻略）\n"
                    "- <ft color=(238,120,0)>ba总力战 寿司 -t 屋外</ft>（Kaiten FX Mk.0 屋外战配队攻略）\n"
                ),
            },
            {
                "func": "活动一图流",
                "trigger_method": "指令",
                "trigger_condition": "ba活动",
                "brief_des": "查询活动攻略图",
                "detail_des": (
                    "发送当前或指定活动一图流攻略图，可能会附带活动特殊机制等\n"
                    "图片作者 B站@夜猫咪喵喵猫\n"
                    " \n"
                    "指令默认发送日服和国际服当前的活动攻略\n"
                    "指令后面跟`日`或`j`开头的文本代表查询日服当前活动攻略，带以`国`或`g`开头的文本同理\n"
                    "跟其他文本则代表指定活动名称\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba活动</ft>\n"
                    "- <ft color=(238,120,0)>ba活动 日</ft>\n"
                    "- <ft color=(238,120,0)>ba活动 温泉浴场</ft>"
                ),
            },
            {
                "func": "综合战术考试一图流",
                "trigger_method": "指令",
                "trigger_condition": "ba综合战术考试",
                "brief_des": "查询综合战术考试攻略图",
                "detail_des": (
                    "发送当前或指定综合战术考试一图流攻略图\n"
                    "图片作者 B站@夜猫咪喵喵猫\n"
                    " \n"
                    "指令默认发送日服和国际服当前的综合战术考试攻略\n"
                    "指令后面跟`日`或`j`开头的文本代表查询日服当前综合战术考试攻略，带以`国`或`g`开头的文本同理\n"
                    "跟整数则代表指定第几个综合战术考试\n"
                    " \n"
                    "p.s. 综合战术考试 和 合同火力演习 其实是一个东西，国际服叫前者，日服叫后者～\n"
                    " \n"
                    "可以用这些指令触发：\n"
                    "- <ft color=(238,120,0)>ba综合战术考试</ft>\n"
                    "- <ft color=(238,120,0)>ba合同火力演习</ft>\n"
                    "- <ft color=(238,120,0)>ba战术考试</ft>\n"
                    "- <ft color=(238,120,0)>ba火力演习</ft>\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba综合战术考试</ft>\n"
                    "- <ft color=(238,120,0)>ba综合战术考试 日</ft>\n"
                    "- <ft color=(238,120,0)>ba综合战术考试 8</ft>"
                ),
            },
            {
                "func": "制造一图流",
                "trigger_method": "指令",
                "trigger_condition": "ba制造",
                "brief_des": "查询制造功能机制图",
                "detail_des": (
                    "发送游戏内制造功能的一图流介绍\n"
                    "图片作者 B站@夜猫咪喵喵猫\n"
                    " \n"
                    "可以用这些指令触发：\n"
                    "- <ft color=(238,120,0)>ba制造</ft>\n"
                    "- <ft color=(238,120,0)>ba制作</ft>\n"
                    "- <ft color=(238,120,0)>ba合成</ft>\n"
                ),
            },
            {
                "func": "国际服千里眼",
                "trigger_method": "指令",
                "trigger_condition": "ba千里眼",
                "brief_des": "查询国际服未来的卡池与活动",
                "detail_des": (
                    "发送当前或指定日期的国际服未来卡池与活动列表\n"
                    "图片作者 B站@夜猫咪喵喵猫\n"
                    " \n"
                    "参数可以指定起始日期以及列表个数，但同时指定时请将日期放在列表个数前面，以空格分隔\n"
                    "参数中含有`全`或`a`时会发送整张前瞻图\n"
                    " \n"
                    "日期格式可以为（Y代表4位数年，m代表月，d代表日）：\n"
                    "- <ft color=(238,120,0)>Y/m/d</ft>；<ft color=(238,120,0)>m/d</ft>\n"
                    "- <ft color=(238,120,0)>Y-m-d</ft>；<ft color=(238,120,0)>m-d</ft>\n"
                    "- <ft color=(238,120,0)>Y年m月d日</ft>；<ft color=(238,120,0)>m月d日</ft>\n"
                    " \n"
                    "可以用这些指令触发：\n"
                    "- <ft color=(238,120,0)>ba国际服千里眼</ft>\n"
                    "- <ft color=(238,120,0)>ba千里眼</ft>\n"
                    "- <ft color=(238,120,0)>ba国际服前瞻</ft>\n"
                    "- <ft color=(238,120,0)>ba前瞻</ft>\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba千里眼</ft>\n"
                    "- <ft color=(238,120,0)>ba千里眼 all</ft>\n"
                    "- <ft color=(238,120,0)>ba千里眼 3</ft>\n"
                    "- <ft color=(238,120,0)>ba千里眼 11/15</ft>\n"
                    "- <ft color=(238,120,0)>ba千里眼 11/15 3</ft>"
                ),
            },
            {
                "func": "学生语音",
                "trigger_method": "指令",
                "trigger_condition": "ba语音",
                "brief_des": "发送学生语音",
                "detail_des": (
                    "从GameKee爬取学生语音并发送\n"
                    "指定关键词时会从匹配结果中随机选择一个语音发送\n"
                    " \n"
                    "可以用这些指令触发：\n"
                    "- <ft color=(238,120,0)>ba语音</ft>\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba语音 忧</ft>\n"
                    "- <ft color=(238,120,0)>ba语音 美游 被cc</ft>"
                ),
            },
            {
                "func": "互动家具总览",
                "trigger_method": "指令",
                "trigger_condition": "ba互动家具",
                "brief_des": "查询互动家具总览图",
                "detail_des": "发送咖啡厅内所有互动家具以及对应学生的总览图\n图片作者 B站@夜猫咪喵喵猫",
            },
            {
                "func": "模拟抽卡",
                "trigger_method": "指令",
                "trigger_condition": "ba抽卡",
                "brief_des": "模拟抽卡",
                "detail_des": (
                    "模拟抽卡\n"
                    "可以使用 <ft color=(238,120,0)>ba切换卡池</ft> 指令来切换卡池\n"
                    "可以指定抽卡次数，需要在1~90之间，默认10\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba抽卡</ft>\n"
                    "- <ft color=(238,120,0)>ba抽卡 20</ft>"
                ),
            },
            {
                "func": "切换卡池",
                "trigger_method": "指令",
                "trigger_condition": "ba切换卡池",
                "brief_des": "设置模拟抽卡的UP池",
                "detail_des": (
                    "设置模拟抽卡功能的UP池角色\n"
                    "默认从当前数据源UP池中轮流切换\n"
                    "当参数为 <ft color=(238,120,0)>常驻</ft> 时，切换到常驻池（没有UP）\n"
                    "可以自定义池子UP角色，支持2星与3星角色，参数中学生名称用空格分隔，支持部分学生别名\n"
                    " \n"
                    "指令示例：\n"
                    "- <ft color=(238,120,0)>ba切换卡池</ft>\n"
                    "- <ft color=(238,120,0)>ba切换卡池 常驻</ft>"
                    "- <ft color=(238,120,0)>ba切换卡池 小桃 小绿</ft>"
                ),
            },
            {
                "func": "抽表情",
                "trigger_method": "指令",
                "trigger_condition": "ba表情",
                "brief_des": "随机发送一个国际服社团聊天表情",
                "detail_des": "随机发送一个国际服社团聊天表情\n来源：解包",
            },
            {
                "func": "随机漫画",
                "trigger_method": "指令",
                "trigger_condition": "ba漫画",
                "brief_des": "随机发送一话官推漫画",
                "detail_des": "随机发送一话BA官推漫画\n来源：GameKee",
            },
            {
                "func": "清空缓存",
                "trigger_method": "超级用户 指令",
                "trigger_condition": "ba清空缓存",
                "brief_des": "清空插件请求缓存",
                "detail_des": (
                    "手动清空插件请求网络缓存下来的数据（正常3小时清空一次）\n"
                    "如果插件出问题了，或者你想提前看到新内容，不妨试试清空一下插件缓存\n"
                    "注：该指令只能由<ft color=(238,120,0)>超级用户</ft>触发\n"
                    " \n"
                    "可以用这些指令触发：\n"
                    "- <ft color=(238,120,0)>ba清空缓存</ft>\n"
                    "- <ft color=(238,120,0)>ba清除缓存</ft>"
                ),
            },
        ],
    },
)



import asyncio
import re
import time
from dataclasses import dataclass
from datetime import datetime
from io import BytesIO
from typing import Any, Dict, List, Union

from PIL import Image
from bs4 import BeautifulSoup, ResultSet, Tag
from nonebot import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot_plugin_htmlrender import get_new_page
from nonebot_plugin_imageutils import BuildImage, text2image
from playwright.async_api import Page

from .resource import RES_CALENDER_BANNER, RES_GRADIENT_BG
from .util import async_req, parse_time_delta


async def game_kee_request(url, **kwargs) -> Union[List, Dict[str, Any]]:
    ret = await async_req(
        url, headers={"game-id": "0", "game-alias": "ba"}, proxy=None, **kwargs
    )
    if ret["code"] != 0:
        raise ConnectionError(ret["msg"])
    return ret["data"]


async def game_kee_get_calender():
    ret: List = await game_kee_request("https://ba.gamekee.com/v1/wiki/index")

    for i in ret:
        if i["module"]["id"] == 12:
            li: list = i["list"]

            now = time.time()
            li = [x for x in li if (now < x["end_at"])]

            li.sort(key=lambda x: x["begin_at"] if now < x["begin_at"] else x["end_at"])
            li.sort(key=lambda x: now < x["begin_at"])
            li.sort(key=lambda x: x["importance"], reverse=True)
            return li


async def game_kee_get_stu_li():
    ret = await game_kee_request("https://ba.gamekee.com/v1/wiki/entry")

    for i in ret["entry_list"]:
        if i["id"] == 23941:

            for ii in i["child"]:
                if ii["id"] == 49443:
                    return {x["name"]: x for x in ii["child"]}


async def game_kee_get_stu_cid_li():
    return {x: y["content_id"] for x, y in (await game_kee_get_stu_li()).items()}


def game_kee_page_url(sid):
    return f"https://ba.gamekee.com/{sid}.html"


async def game_kee_get_page(url):
    async with get_new_page() as page:  # type:Page
        await page.goto(url, timeout=60 * 1000)

        # 删掉header
        await page.add_script_tag(
            content='document.getElementsByClassName("wiki-header")'
            ".forEach((v)=>{v.remove()})"
        )

        # 展开折叠的语音
        folds = await page.query_selector_all('xpath=//div[@class="fold-table-btn"]')
        for i in folds:
            try:
                await i.click()
            except:
                pass

        return await (
            await page.query_selector('xpath=//div[@class="wiki-detail-body"]')
        ).screenshot()


async def game_kee_calender():
    ret = await game_kee_get_calender()
    if not ret:
        return "没有获取到GameKee日程表数据"

    pic = await game_kee_get_calender_page(ret)
    return MessageSegment.image(pic)


async def game_kee_get_calender_page(ret, has_pic=True):
    now = datetime.now()

    async def draw(it: dict):
        _p = None
        if has_pic and (_p := it.get("picture")):
            try:
                _p = (
                    BuildImage.open(BytesIO(await async_req(f"https:{_p}", raw=True)))
                    .resize_width(1290)
                    .circle_corner(15)
                )
            except:
                logger.exception("下载日程表图片失败")

        begin = datetime.fromtimestamp(it["begin_at"])
        end = datetime.fromtimestamp(it["end_at"])
        started = begin <= now
        time_remain = (end if started else begin) - now
        dd, hh, mm, ss = parse_time_delta(time_remain)

        # logger.debug(f'{it["title"]} | {started} | {time_remain}')

        title_p = text2image(
            f'[b]{it["title"]}[/b]', "#ffffff00", max_width=1290, fontsize=65
        )
        time_p = text2image(
            f"{begin} ~ {end}", "#ffffff00", max_width=1290, fontsize=40
        )
        desc_p = (
            text2image(
                desc.replace("<br>", ""),
                "#ffffff00",
                max_width=1290,
                fontsize=40,
            )
            if (desc := it["description"])
            else None
        )
        remain_p = text2image(
            f"剩余 [color=#fc6475]{dd}[/color] 天 [color=#fc6475]{hh}[/color] 时 "
            f"[color=#fc6475]{mm}[/color] 分 [color=#fc6475]{ss}[/color] 秒"
            f'{"结束" if started else "开始"}',
            "#ffffff00",
            max_width=1290,
            fontsize=50,
        )

        h = (
            100
            + (title_p.height + 25)
            + (time_p.height + 25)
            + (_p.height + 25 if _p else 0)
            + (desc_p.height + 25 if desc_p else 0)
            + remain_p.height
        )
        img = BuildImage.new("RGBA", (1400, h), (255, 255, 255, 70)).draw_rectangle(
            (0, 0, 10, h), "#fc6475" if it["importance"] else "#4acf75"
        )

        if not started:
            img.draw_rectangle((1250, 0, 1400, 60), "gray")
            img.draw_text((1250, 0, 1400, 60), "未开始", 50, fill="white")

        ii = 50
        img.paste(title_p, (60, ii), True)
        ii += title_p.height + 25
        img.paste(time_p, (60, ii), True)
        ii += time_p.height + 25
        if _p:
            img.paste(_p, (60, ii), True)
            ii += _p.height + 25
        if desc_p:
            img.paste(desc_p, (60, ii), True)
            ii += desc_p.height + 25
        img.paste(remain_p, (60, ii), True)
        # img = img.circle_corner(15)

        return img

    pics: List[BuildImage] = await asyncio.gather(  # type: ignore
        *[draw(x) for x in ret]
    )

    bg_w = 1500
    bg_h = 200 + sum([x.height + 50 for x in pics])
    bg = (
        BuildImage.new("RGBA", (bg_w, bg_h))
        .paste(RES_CALENDER_BANNER.copy().resize((1500, 150)))
        .draw_text(
            (50, 0, 1480, 150),
            "GameKee丨活动日程",
            100,
            weight="bold",
            fill="#ffffff",
            halign="left",
        )
        .paste(
            RES_GRADIENT_BG.copy().resize((1500, bg_h - 150), resample=Image.NEAREST),
            (0, 150),
        )
    )

    index = 200
    for p in pics:
        bg.paste(p.circle_corner(10), (50, index), True)
        index += p.height + 50

    return bg.save_jpg()


async def game_kee_grab_l2d(cid):
    r: dict = await game_kee_request(f"https://ba.gamekee.com/v1/content/detail/{cid}")
    r: str = r["content"]

    i = r.find('<div class="input-wrapper">官方介绍</div>')
    i = r.find('class="slide-item" data-index="2"', i)
    ii = r.find('data-index="3"', i)

    r: str = r[i:ii]

    img = re.findall('data-real="([^"]*)"', r)

    return [f"https:{x}" for x in img]


@dataclass()
class GameKeeVoice:
    title: str
    jp: str
    cn: str
    url: str


async def game_kee_get_voice(cid) -> List[GameKeeVoice]:
    wiki_html = (
        await game_kee_request(f"https://ba.gamekee.com/v1/content/detail/{cid}")
    )["content"]
    bs = BeautifulSoup(wiki_html, "lxml")
    audios = bs.select(".mould-table>tbody>tr>td>div>div>audio")

    parsed: List[GameKeeVoice] = []
    for au in audios:
        url: str = au["src"]
        if not url.startswith("http"):
            url = f"https:{url}"

        tr1: Tag = au.parent.parent.parent.parent
        tds: ResultSet[Tag] = tr1.find_all("td")
        title = tds[0].text.strip()
        jp = "\n".join(tds[2].stripped_strings)

        tr2 = tr1.next_sibling
        cn = "\n".join(tr2.stripped_strings)
        parsed.append(GameKeeVoice(title, jp, cn, url))

    return parsed



from typing import Optional

from nonebot import get_driver
from pydantic import BaseModel


class Cfg(BaseModel):
    proxy: Optional[str] = None


config = Cfg.parse_obj(get_driver().config.dict())



from pathlib import Path

SCHALE_URL = "https://lonqie.github.io/SchaleDB/"
MIRROR_SCHALE_URL = "http://schale.lgc2333.top/"

BAWIKI_DB_URL = "https://bawiki.lgc2333.top/"

RES_PATH = Path(__file__).parent / "res"

DATA_PATH = Path.cwd() / "data" / "BAWiki"
if not DATA_PATH.exists():
    DATA_PATH.mkdir(parents=True)



import datetime
import json
from copy import deepcopy
from typing import Dict, Iterator, List, TypeVar, Union

from PIL import Image, ImageOps
from aiohttp import ClientSession
from nonebot.adapters.onebot.v11 import Message

from .config import config

_T = TypeVar("_T")

req_cache = {}


def format_timestamp(t):
    return datetime.datetime.fromtimestamp(t).strftime("%Y-%m-%d %H:%M:%S")


def recover_alia(origin: str, alia_dict: Dict[str, List[str]]):
    origin = replace_brackets(origin).strip()
    origin_ = origin.lower()

    # 精确匹配
    for k, li in alia_dict.items():
        if origin_ in li or origin_ == k:
            return k

    # 没找到，模糊匹配
    origin_ = origin.replace(" ", "")
    for k, li in alia_dict.items():
        li = [x.replace(" ", "") for x in ([k] + li)]
        for v in li:
            if origin_ in v:
                return k

    return origin


def parse_time_delta(t: datetime.timedelta):
    mm, ss = divmod(t.seconds, 60)
    hh, mm = divmod(mm, 60)
    dd = t.days or 0
    return dd, hh, mm, ss


def img_invert_rgba(im: Image.Image):
    # https://stackoverflow.com/questions/2498875/how-to-invert-colors-of-image-with-pil-python-imaging
    r, g, b, a = im.split()
    rgb_image = Image.merge("RGB", (r, g, b))
    inverted_image = ImageOps.invert(rgb_image)
    r2, g2, b2 = inverted_image.split()
    final_transparent_image = Image.merge("RGBA", (r2, g2, b2, a))
    return final_transparent_image


async def async_req(
    url,
    is_json=True,
    raw=False,
    ignore_cache=False,
    proxy=config.proxy,
    method="GET",
    session: ClientSession = None,
    **kwargs,
) -> Union[str, bytes, dict, list]:
    if (not ignore_cache) and (c := req_cache.get(url)):
        return c

    async with (session or ClientSession()) as c:
        async with c.request(method, url, **kwargs, proxy=proxy) as r:
            ret = (await r.read()) if raw else (await r.text())
            if is_json and (not raw):
                ret = json.loads(ret)
            req_cache[url] = ret
            return ret


def clear_req_cache():
    req_cache.clear()


def replace_brackets(original: str):
    return original.replace("（", "(").replace("）", "(")


def splice_msg(msgs):
    im = Message()
    for i, v in enumerate(msgs):
        if isinstance(v, str) and (not i == 0):
            v = f"\n{v}"
        im += v
    return im


def split_list(li: Iterator[_T], length: int) -> List[List[_T]]:
    latest = []
    tmp = []
    for n, i in enumerate(li):
        tmp.append(i)
        if (n + 1) % length == 0:
            latest.append(deepcopy(tmp))
            tmp.clear()
    if tmp:
        latest.append(deepcopy(tmp))
    return latest



import asyncio
import datetime
import math
import random
from argparse import Namespace
from dataclasses import dataclass
from io import BytesIO
from typing import Dict, List, Optional

from nonebot import logger, on_command, on_shell_command
from nonebot.adapters.onebot.v11 import (
    ActionFailed,
    Message,
    MessageEvent,
    MessageSegment,
)
from nonebot.exception import FinishedException, ParserExit
from nonebot.internal.matcher import Matcher
from nonebot.params import CommandArg, ShellCommandArgs
from nonebot.permission import SUPERUSER
from nonebot.rule import ArgumentParser
from nonebot_plugin_apscheduler import scheduler
from nonebot_plugin_imageutils import BuildImage

from .const import BAWIKI_DB_URL, SCHALE_URL
from .data_bawiki import (
    MangaDict,
    db_get,
    db_get_emoji,
    db_get_event_alias,
    db_get_extra_l2d_list,
    db_get_gacha_data,
    db_get_manga,
    db_get_raid_alias,
    db_get_stu_alias,
    db_get_terrain_alias,
    db_global_future,
    db_wiki_craft,
    db_wiki_event,
    db_wiki_furniture,
    db_wiki_raid,
    db_wiki_stu,
    db_wiki_time_atk,
    recover_stu_alia,
    schale_to_gamekee,
)
from .data_gamekee import (
    GameKeeVoice,
    game_kee_calender,
    game_kee_get_page,
    game_kee_get_stu_cid_li,
    game_kee_get_voice,
    game_kee_grab_l2d,
    game_kee_page_url,
)
from .data_schaledb import (
    draw_fav_li,
    find_current_event,
    schale_calender,
    schale_get_common,
    schale_get_stu_dict,
    schale_get_stu_info,
)
from .gacha import gacha
from .util import async_req, clear_req_cache, recover_alia, splice_msg


@dataclass()
class GachaPool:
    name: str
    pool: List[int]
    index: Optional[int] = None


gacha_pool_index: Dict[str, GachaPool] = {}


@scheduler.scheduled_job("interval", hours=3)
async def _():
    clear_req_cache()


h_clear_cache = on_command("ba清空缓存", aliases={"ba清除缓存"}, permission=SUPERUSER)


@h_clear_cache.handle()
async def _(matcher: Matcher):
    clear_req_cache()
    await matcher.finish("缓存已清空～")


handler_calender = on_command("ba日程表")


@handler_calender.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg: str = arg.extract_plain_text()

    await matcher.send("正在绘制图片，请稍等")
    try:
        if "schale" in (arg := arg.lower()):
            arg = arg.replace("schale", "").strip()
            servers = []

            if (not arg) or ("日" in arg) or ("j" in arg):
                servers.append(0)
            if (not arg) or ("国" in arg) or ("g" in arg):
                servers.append(1)

            await asyncio.gather(
                *[
                    matcher.send(x)
                    for x in (
                        await asyncio.gather(*[schale_calender(x) for x in servers])
                    )
                ]
            )
            await matcher.finish()
        else:
            await matcher.finish(await game_kee_calender())
    except (FinishedException, ActionFailed):
        raise
    except:
        logger.exception("绘制日程表图片出错")
        return await matcher.finish("绘制日程表图片出错，请检查后台输出")


async def send_wiki_page(sid, matcher: Matcher):
    url = game_kee_page_url(sid)
    await matcher.send(f"请稍等，正在截取Wiki页面……\n{url}")

    try:
        img = await game_kee_get_page(url)
    except:
        logger.exception(f"截取wiki页面出错 {url}")
        return await matcher.finish("截取页面出错，请检查后台输出")

    await matcher.finish(MessageSegment.image(img))


stu_schale = on_command("ba学生图鉴")


@stu_schale.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip()
    if not arg:
        return await matcher.finish("请提供学生名称")

    try:
        ret = await schale_get_stu_dict()
    except:
        logger.exception("获取学生列表出错")
        return await matcher.finish("获取学生列表表出错，请检查后台输出")

    if not ret:
        return await matcher.finish("没有获取到学生列表数据")

    if not (data := ret.get(await recover_stu_alia(arg))):
        return await matcher.finish("未找到该学生")

    stu_name = data["PathName"]
    await matcher.send(f"请稍等，正在截取SchaleDB页面～\n" f"{SCHALE_URL}?chara={stu_name}")

    try:
        img = MessageSegment.image(await schale_get_stu_info(stu_name))
    except:
        logger.exception(f"截取schale db页面出错 chara={stu_name}")
        return await matcher.finish("截取页面出错，请检查后台输出")

    await matcher.finish(img)


stu_rank = on_command("ba学生评价", aliases={"ba角评"})


@stu_rank.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip()
    if not arg:
        return await matcher.finish("请提供学生名称")

    if arg == "总览" or arg == "全部" or arg.lower() == "all":
        arg = "all"
    else:
        arg = await recover_stu_alia(arg)

    try:
        im = await db_wiki_stu(arg)
    except:
        logger.exception(f"获取角评出错")
        return await matcher.finish("获取角评出错，请检查后台输出")

    await matcher.finish(im)


stu_wiki = on_command("ba学生wiki", aliases={"ba学生Wiki", "ba学生WIKI"})


@stu_wiki.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip()
    if not arg:
        return await matcher.finish("请提供学生名称")

    try:
        ret = await game_kee_get_stu_cid_li()
    except:
        logger.exception("获取学生列表出错")
        return await matcher.finish("获取学生列表出错，请检查后台输出")

    if not ret:
        return await matcher.finish("没有获取到学生列表数据")

    if not (sid := ret.get(await recover_stu_alia(arg, True))):
        return await matcher.finish("未找到该学生")

    await send_wiki_page(sid, matcher)


fav = on_command("ba好感度", aliases={"ba羁绊", "bal2d", "baL2D", "balive2d", "baLive2D"})


@fav.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    async def get_l2d(stu_name):
        if r := (await db_get_extra_l2d_list()).get(stu_name):
            return f"{BAWIKI_DB_URL}{r}"

        return await game_kee_grab_l2d((await game_kee_get_stu_cid_li()).get(stu_name))

    arg = arg.extract_plain_text().strip()
    if not arg:
        return await matcher.finish("请提供学生名称或所需的羁绊等级")

    # 好感度等级
    if arg.isdigit():
        arg = int(arg)
        if arg > 9:
            return await matcher.finish("学生解锁L2D最高只需要羁绊等级9")
        if arg < 1:
            return await matcher.finish("学生解锁L2D最低只需要羁绊等级1")

        try:
            p = await draw_fav_li(arg)
        except:
            logger.exception("绘制图片出错")
            return await matcher.finish("绘制图片出错，请检查后台输出")

        return await matcher.finish(p)

    # 学生名称
    arg = await recover_stu_alia(arg)

    try:
        ret = await schale_get_stu_dict()
    except:
        logger.exception("获取学生列表出错")
        return await matcher.finish("获取学生列表表出错，请检查后台输出")

    if stu := ret.get(arg):
        if not (lvl := stu["MemoryLobby"]):
            return await matcher.finish("该学生没有L2D")

        im = MessageSegment.text(f'{stu["Name"]} 在羁绊等级 {lvl[0]} 时即可解锁L2D\nL2D预览：')
        if p := await get_l2d(await schale_to_gamekee(arg)):
            im += [MessageSegment.image(await async_req(x, raw=True)) for x in p]
        else:
            im += (
                "没找到该学生的L2D看板\n"
                "可能原因：\n"
                "- GameKee页面爬取不到角色L2D图片\n"
                "- GameKee和插件没有收录该学生的L2D\n"
            )
        return await matcher.finish(im)

    return await matcher.finish("未找到学生")


raid_wiki_parser = ArgumentParser("ba总力战")
raid_wiki_parser.add_argument(
    "name", nargs="?", default=None, help="总力战Boss名称，不指定默认取当前服务器总力战Boss"
)
raid_wiki_parser.add_argument(
    "-s",
    "--server",
    nargs="*",
    help="服务器名称，`j`或`日`代表日服，`g`或`国`代表国际服，可指定多个，默认全选",
    default=["j", "g"],
)
raid_wiki_parser.add_argument("-t", "--terrain", help="指定总力战环境，不指定默认全选，不带Boss名称该参数无效")
raid_wiki_parser.add_argument(
    "-w", "--wiki", action="store_true", help="发送该总力战Boss的技能机制而不是配队推荐"
)

raid_wiki = on_shell_command("ba总力战", parser=raid_wiki_parser)


@raid_wiki.handle()
async def _(matcher: Matcher, foo: ParserExit = ShellCommandArgs()):
    im = ""
    if foo.status != 0:
        im = "参数错误\n"
    await matcher.finish(f"{im}{foo.message}")


@raid_wiki.handle()
async def _(matcher: Matcher, args: Namespace = ShellCommandArgs()):
    if not args.server:
        await matcher.finish(f"请指定server参数")

    server = set()
    for s in args.server:
        if ("日" in s) or ("j" in s):
            server.add(0)
        elif ("国" in s) or ("g" in s):
            server.add(1)
    server = list(server)
    server.sort()

    tasks = []
    if not args.name:
        try:
            common = await schale_get_common()
            for s in server:
                raid = common["regions"][s]["current_raid"]
                if (r := find_current_event(raid)) and (raid := r[0]["raid"]) < 1000:
                    tasks.append(
                        db_wiki_raid(raid, [s], args.wiki, r[0].get("terrain"))
                    )
        except:
            logger.exception(f"获取当前总力战失败")
            return await matcher.finish(f"获取当前总力战失败")

        if not tasks:
            return await matcher.finish(f"目前服务器没有正在进行的总力战，请手动指定")
    else:
        tasks.append(
            db_wiki_raid(
                recover_alia(args.name, await db_get_raid_alias()),
                server,
                args.wiki,
                (
                    recover_alia(args.terrain, await db_get_terrain_alias())
                    if args.terrain
                    else None
                ),
            )
        )

    try:
        ret = await asyncio.gather(*tasks)
    except:
        logger.exception("获取总力战wiki失败")
        return await matcher.finish(f"获取图片失败，请检查后台输出")

    await matcher.finish(splice_msg(ret))


event_wiki = on_command("ba活动")


@event_wiki.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().lower().strip()

    server = []
    if arg.startswith("日") or arg.startswith("j") or (not arg):
        server.append(0)
    if arg.startswith("国") or arg.startswith("g") or (not arg):
        server.append(1)

    events = []
    if server:
        try:
            common = await schale_get_common()
            for s in server:
                ev = common["regions"][s]["current_events"]
                if e := find_current_event(ev):
                    events.append((e[0]["event"]) % 10000)
        except:
            logger.exception(f"获取当前活动失败")
            return await matcher.finish(f"获取当前活动失败")

        if not events:
            await matcher.finish("当前服务器没有正在进行的活动")

    else:
        events.append(recover_alia(arg, await db_get_event_alias()))

    try:
        ret = await asyncio.gather(*[db_wiki_event(x) for x in events])
    except:
        logger.exception("获取活动wiki出错")
        return await matcher.finish("获取图片出错，请检查后台输出")

    await matcher.finish(splice_msg(ret))


time_atk_wiki = on_command("ba综合战术考试", aliases={"ba合同火力演习", "ba战术考试", "ba火力演习"})


@time_atk_wiki.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().lower().strip()

    server = []
    if arg.startswith("日") or arg.startswith("j") or (not arg):
        server.append(0)
    if arg.startswith("国") or arg.startswith("g") or (not arg):
        server.append(1)

    events = []
    if server:
        try:
            common = await schale_get_common()
            for s in server:
                raid = common["regions"][s]["current_raid"]
                if (r := find_current_event(raid)) and (raid := r[0]["raid"]) >= 1000:
                    events.append(raid)
        except:
            logger.exception(f"获取当前综合战术考试失败")
            return await matcher.finish(f"获取当前综合战术考试失败")

        if not events:
            await matcher.finish("当前服务器没有正在进行的综合战术考试")

    else:
        if (not str(arg).isdigit()) or ((arg := int(arg)) < 1):
            await matcher.finish("综合战术考试ID需为整数，从1开始，代表第1个综合战术考试")
        events.append(arg)

    try:
        ret = await asyncio.gather(*[db_wiki_time_atk(x) for x in events])
    except:
        logger.exception("获取综合战术考试wiki出错")
        return await matcher.finish("获取图片出错，请检查后台输出")

    await matcher.finish(splice_msg(ret))


craft_wiki = on_command("ba制造", aliases={"ba合成", "ba制作"})


@craft_wiki.handle()
async def _(matcher: Matcher):
    try:
        im = await db_wiki_craft()
    except:
        logger.exception("获取合成wiki图片错误")
        return await matcher.finish("获取图片失败，请检查后台输出")

    await matcher.finish(im)


global_future = on_command("ba国际服千里眼", aliases={"ba千里眼", "ba国际服前瞻", "ba前瞻"})


@global_future.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    args = arg.extract_plain_text().strip()
    if "全" in args or "a" in args:
        await matcher.finish(await db_global_future(all_img=True))

    args = args.split()
    num = 1
    date = None
    if (args_len := len(args)) == 1:
        if args[0].isdigit():
            num = args[0]
        else:
            date = args[0]
    elif args_len > 1:
        date = args[0].strip()
        num = args[-1].strip()

    if date:
        parsed_date = None
        for f in ["%Y/%m/%d", "%Y-%m-%d", "%Y年%m月%d日", "%m/%d", "%m-%d", "%m月%d日"]:
            try:
                parsed_date = datetime.datetime.strptime(date.replace(" ", ""), f)
                break
            except ValueError:
                pass
        if not parsed_date:
            await matcher.finish("日期格式不正确！")
        date = parsed_date
        if date.year == 1900:
            now = datetime.datetime.now().replace(
                hour=0, minute=0, second=0, microsecond=0
            )
            date = date.replace(year=now.year)
            if date < now:
                date = date.replace(year=now.year + 1)

    if isinstance(num, str):
        if (not num.isdigit()) or (num := int(num)) < 1:
            await matcher.finish("前瞻项目数量格式不正确！")

    await matcher.finish(await db_global_future(date, num))


furniture_wiki = on_command("ba互动家具")


@furniture_wiki.handle()
async def _(matcher: Matcher):
    try:
        im = await db_wiki_furniture()
    except:
        logger.exception("获取互动家具wiki图片错误")
        return await matcher.finish("获取图片失败，请检查后台输出")

    await matcher.finish(im)


voice = on_command("ba语音")


@voice.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip()
    if not arg:
        return await matcher.finish("请提供学生名称")

    arg = arg.split()
    arg_len = len(arg)
    name = " ".join(arg[:-1]) if arg_len > 1 else arg[0]
    v_type = arg[-1].strip().lower() if arg_len > 1 else None

    try:
        ret = await game_kee_get_stu_cid_li()
    except:
        logger.exception("获取学生列表出错")
        return await matcher.finish("获取学生列表出错，请检查后台输出")

    if not ret:
        return await matcher.finish("没有获取到学生列表数据")

    try:
        org_stu_name = await recover_stu_alia(name, True)
        stu_name = await schale_to_gamekee(org_stu_name)
    except:
        logger.exception("还原学生别名失败")
        return await matcher.finish("还原学生别名失败，请检查后台输出")

    if not (sid := ret.get(stu_name)):
        return await matcher.finish("未找到该学生")

    voices = await game_kee_get_voice(sid)
    if v_type:
        voices = [x for x in voices if v_type in x.title.lower()]
    if not voices:
        await matcher.finish("没找到符合要求的语音捏")

    v: GameKeeVoice = random.choice(voices)
    v_data = await async_req(v.url, raw=True)

    im = [f"学生 {org_stu_name} 语音 {v.title}\n-=-=-=-=-=-=-=-"]
    if v.jp:
        im.append(v.jp)
    if v.cn:
        im.append(v.cn)
    await matcher.send("\n".join(im))
    await matcher.send(MessageSegment.record(v_data))


def get_1st_pool(data: dict) -> Optional[GachaPool]:
    if not data:
        return

    pool_data = data["current_pools"]
    pool = pool_data[0]
    pool_obj = GachaPool(name=pool["name"], pool=pool["pool"], index=0)
    return pool_obj


change_pool = on_command("ba切换卡池")


@change_pool.handle()
async def _(matcher: Matcher, event: MessageEvent, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip().lower()
    qq = event.get_user_id()

    if arg:
        if "常驻" in arg:
            current = GachaPool(name="常驻池", pool=[])
        else:
            pool = []
            try:
                stu_li = await schale_get_stu_dict()
                stu_alias = await db_get_stu_alias()
            except:
                logger.exception("获取学生列表或别名失败")
                return await matcher.finish("获取学生列表或别名失败，请检查后台输出")

            for i in arg.split():
                if not (stu := stu_li.get(recover_alia(i, stu_alias))):
                    await matcher.finish(f"未找到学生 {i}")
                if stu["StarGrade"] == 1:
                    await matcher.finish("不能UP一星角色")
                pool.append(stu)

            current = GachaPool(
                name=f"自定义卡池（{'、'.join([x['Name'] for x in pool])}）",
                pool=[x["Id"] for x in pool],
            )

    else:
        try:
            gacha_data = await db_get_gacha_data()
        except:
            logger.exception("获取抽卡基本数据失败")
            return await matcher.finish("获取抽卡基本数据失败，请检查后台输出")

        pool_data = gacha_data["current_pools"]
        if not pool_data:
            await matcher.finish("当前没有可切换的卡池")

        pool_obj = gacha_pool_index.get(qq) or get_1st_pool(gacha_data)
        if not pool_obj:
            await matcher.finish("当前没有UP池可供切换")

        if (index := pool_obj.index) is not None:
            index += 1
            if len(pool_data) <= index:
                index = 0
        else:
            index = 0

        pool = pool_data[index]
        current = GachaPool(name=pool["name"], pool=pool["pool"], index=index)

    gacha_pool_index[qq] = current
    await matcher.finish(f"已切换到卡池 {current.name}")


gacha_once = on_command("ba抽卡")


@gacha_once.handle()
async def _(matcher: Matcher, event: MessageEvent, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip().lower()

    gacha_times = 10
    if arg:
        if (not arg.isdigit()) or (not (1 <= (gacha_times := int(arg)) <= 90)):
            await matcher.finish("请输入有效的抽卡次数，在1~90之间")

    try:
        gacha_data = await db_get_gacha_data()
    except:
        logger.exception("获取抽卡基本数据失败")
        return await matcher.finish("获取抽卡基本数据失败，请检查后台输出")

    pool_obj = gacha_pool_index.get(qq := event.get_user_id()) or get_1st_pool(
        gacha_data
    )
    if not pool_obj:
        await matcher.finish("目前没有UP池，请使用[ba切换卡池]指令来切换到常驻池或组建一个自定义UP池")

    try:
        img = []
        for _ in range(math.ceil(gacha_times / 10)):
            img.append(
                await gacha(
                    qq,
                    10 if gacha_times >= 10 else gacha_times,
                    gacha_data,
                    pool_obj.pool,
                )
            )
            gacha_times -= 10
    except:
        logger.exception("抽卡错误")
        return await matcher.finish("抽卡出错了，请检查后台输出")

    await matcher.finish(MessageSegment.at(qq) + f"当前抽取卡池：{pool_obj.name}" + img)


random_emoji = on_command("ba表情")


@random_emoji.handle()
async def _(matcher: Matcher):
    try:
        emojis = await db_get_emoji()
        emo = await db_get(random.choice(emojis), True)
    except:
        logger.exception("获取表情失败")
        return await matcher.finish("获取表情失败，请检查后台输出")
    await matcher.finish(MessageSegment.image(emo))


random_manga = on_command("ba漫画")


@random_manga.handle()
async def _(matcher: Matcher):
    async def get_pic(url):
        p = await async_req(url, raw=True)
        if url.endswith(".webp"):
            p = BuildImage.open(BytesIO(p)).save_png()
        return p

    try:
        manga: MangaDict = random.choice(await db_get_manga())
        pics = await asyncio.gather(*[get_pic(x) for x in manga["pics"]])
    except:
        logger.exception("获取漫画失败")
        return await matcher.finish("获取漫画失败，请检查后台输出")

    await matcher.finish(
        Message()
        + f'{manga["title"]}\n-=-=-=-=-=-=-=-\n{manga["detail"]}'
        + [MessageSegment.image(x) for x in pics]
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_bawiki']

package_data = \
{'': ['*'], 'nonebot_plugin_bawiki': ['res/*']}

install_requires = \
['Pillow>=9.2.0,<10.0.0',
 'aiohttp>=3.8.3,<4.0.0',
 'beautifulsoup4>=4.11.1,<5.0.0',
 'lxml>=4.9.1,<5.0.0',
 'nonebot-adapter-onebot>=2.1.1',
 'nonebot-plugin-apscheduler>=0.2.0,<0.3.0',
 'nonebot-plugin-htmlrender>=0.2.0.1,<0.3.0.0',
 'nonebot-plugin-imageutils>=0.1.13.4,<0.2.0.0',
 'nonebot2>=2.0.0-beta.5',
 'pillow>=9.3.0,<10.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-bawiki',
    'version': '0.6.3',
    'description': 'A nonebot2 plugin for Blue Archive.',
    'long_description': '<!-- markdownlint-disable MD033 MD036 MD041 -->\n\n<div align="center">\n  <a href="https://v2.nonebot.dev/store"><img src="https://raw.githubusercontent.com/lgc2333/nonebot-plugin-bawiki/master/readme/nonebot-plugin-bawiki.png" width="200" height="200" alt="BAWiki"></a>\n</div>\n\n<div align="center">\n\n# NoneBot-Plugin-BAWiki\n\n_✨ 基于 NoneBot2 的碧蓝档案 Wiki 插件 ✨_\n\n<a href="./LICENSE">\n    <img src="https://img.shields.io/github/license/lgc2333/nonebot-plugin-bawiki.svg" alt="license">\n</a>\n<a href="https://pypi.python.org/pypi/nonebot-plugin-bawiki">\n    <img src="https://img.shields.io/pypi/v/nonebot-plugin-bawiki.svg" alt="pypi">\n</a>\n<img src="https://img.shields.io/badge/python-3.8+-blue.svg" alt="python">\n<a href="https://pypi.python.org/pypi/nonebot-plugin-bawiki">\n    <img src="https://img.shields.io/pypi/dm/nonebot-plugin-bawiki" alt="pypi download">\n</a>\n<a href="https://wakatime.com/badge/user/b61b0f9a-f40b-4c82-bc51-0a75c67bfccf/project/371bbbba-9dba-4e40-883c-72b688876575">\n    <img src="https://wakatime.com/badge/user/b61b0f9a-f40b-4c82-bc51-0a75c67bfccf/project/371bbbba-9dba-4e40-883c-72b688876575.svg" alt="wakatime">\n</a>\n\n</div>\n\n## 💬 前言\n\n诚邀各位帮忙更新插件数据源仓库！能帮这个小小插件贡献微薄之力，鄙人感激不尽！！\n\n[点击跳转 bawiki-data](https://github.com/lgc2333/bawiki-data)\n\n修改后提交 Pull Request 即可！\n\n## 📖 介绍\n\n一个碧蓝档案的 Wiki 插件，主要数据来源为 [GameKee](https://ba.gamekee.com/) 与 [SchaleDB](https://lonqie.github.io/SchaleDB/)  \n插件灵感来源：[ba_calender](https://f.xiaolz.cn/forum.php?mod=viewthread&tid=145)\n\n## 💿 安装\n\n<details open>\n<summary>【推荐】使用 nb-cli 安装</summary>\n在 nonebot2 项目的根目录下打开命令行, 输入以下指令即可安装\n\n    nb plugin install nonebot-plugin-bawiki\n\n</details>\n\n<details>\n<summary>使用包管理器安装</summary>\n在 nonebot2 项目的插件目录下, 打开命令行, 根据你使用的包管理器, 输入相应的安装命令\n\n<details>\n<summary>pip</summary>\n\n    pip install nonebot-plugin-bawiki\n\n</details>\n<details>\n<summary>pdm</summary>\n\n    pdm add nonebot-plugin-bawiki\n\n</details>\n<details>\n<summary>poetry</summary>\n\n    poetry add nonebot-plugin-bawiki\n\n</details>\n<details>\n<summary>conda</summary>\n\n    conda install nonebot-plugin-bawiki\n\n</details>\n\n打开 nonebot2 项目的 `bot.py` 文件, 在其中写入\n\n    nonebot.load_plugin(\'nonebot_plugin_bawiki\')\n\n</details>\n\n<details>\n<summary>从 github 安装</summary>\n在 nonebot2 项目的插件目录下, 打开命令行, 输入以下命令克隆此储存库\n\n    git clone https://github.com/lgc2333/nonebot-plugin-bawiki.git\n\n打开 nonebot2 项目的 `bot.py` 文件, 在其中写入\n\n    nonebot.load_plugin(\'src.plugins.nonebot_plugin_bawiki\')\n\n</details>\n\n## ⚙️ 配置\n\n在 nonebot2 项目的`.env`文件中添加下表中的配置\n\n| 配置项  | 必填 | 默认值 |                         说明                          |\n| :-----: | :--: | :----: | :---------------------------------------------------: |\n| `PROXY` |  否  | `None` | 访问`SchaleDB`、`bawiki-data`的 json 数据时使用的代理 |\n\n## 🎉 使用\n\n### 指令表\n\n兼容 [nonebot-plugin-PicMenu](https://github.com/hamo-reid/nonebot_plugin_PicMenu)\n\n见[这里](https://github.com/lgc2333/nonebot-plugin-bawiki/blob/master/nonebot_plugin_bawiki/__init__.py#L17)\n\n待更新\n\n<!--\n### 效果图\n\n<details>\n<summary>长图，点击展开</summary>\n\n![example](https://raw.githubusercontent.com/lgc2333/nonebot-plugin-bawiki/master/readme/example.png)\n![example2](https://raw.githubusercontent.com/lgc2333/nonebot-plugin-bawiki/master/readme/example2.png)\n\n</details>\n-->\n\n## 📞 联系\n\nQQ：3076823485  \nTelegram：[@lgc2333](https://t.me/lgc2333)  \n吹水群：[1105946125](https://jq.qq.com/?_wv=1027&k=Z3n1MpEp)  \n邮箱：<lgc2333@126.com>\n\n## 💡 鸣谢\n\n### [RainNight0](https://github.com/RainNight0)\n\n- 日程表 html 模板提供（已弃用）\n\n### `bawiki-data`数据源贡献列表\n\n- 见 [bawiki-data](http://github.com/lgc2333/bawiki-data)\n\n## 💰 赞助\n\n感谢大家的赞助！你们的赞助将是我继续创作的动力！\n\n- [爱发电](https://afdian.net/@lgc2333)\n- <details>\n    <summary>赞助二维码（点击展开）</summary>\n\n  ![讨饭](https://raw.githubusercontent.com/lgc2333/ShigureBotMenu/master/src/imgs/sponsor.png)\n\n  </details>\n\n## 📝 更新日志\n\n### 0.6.3\n\n- 使用 `require` 加载依赖插件\n\n### 0.6.2\n\n- 修改日程表、羁绊查询的图片背景\n- 加上日程表条目的圆角\n- 更改 GameKee 日程表的排序方式\n\n### 0.6.1\n\n- 修复一处 Py 3.8 无法运行的代码\n\n### 0.6.0\n\n- 新指令 `ba抽卡` `ba切换卡池` `ba表情` `ba漫画`\n- 更改 SchaleDB 日程表触发单国际服的指令判断（由包含`国际服`改为包含`国`）\n\n### 0.5.2\n\n- 新指令`ba语音`\n- 修复`ba综合战术考试`的一些问题\n\n### 0.5.1\n\n- 新指令`ba互动家具`\n- `ba国际服千里眼`指令的日期参数如果小于当前日期则会将日期向前推一年\n- `ba日程表`的 SchaleDB 源如果没获取到数据则不会绘画那一部分\n- `ba国际服千里眼`日期匹配 bug 修复\n\n### 0.5.0\n\n- 新数据源 [bawiki-data](http://github.com/lgc2333/bawiki-data)\n- 新指令`ba角评`；`ba总力战`；`ba活动`；`ba综合战术考试`；`ba制造`；`ba国际服千里眼`；`ba清空缓存`\n- 将`bal2d`指令改为`ba羁绊`别名\n- 将`ba日程表`指令从网页截图改为 Pillow 画图；并修改了指令的参数解析方式\n- 更改了`ba羁绊`指令的画图方式及底图\n- 更改学生别名的匹配方式\n- 学生别名等常量现在从 [bawiki-data](http://github.com/lgc2333/bawiki-data) 在线获取\n- 新增请求接口的缓存机制，每3小时清空一次缓存\n- 新增`PROXY`配置项\n- 更改三级菜单排版\n\n### 0.4.2\n\n- `ba羁绊` `baL2D` 的 L2D 预览图改为实时从 GameKee 抓取\n\n### 0.4.1\n\n- 优化带括号学生名称的别名匹配\n\n### 0.4.0\n\n- `ba日程表`的`SchaleDB`数据源\n- `ba学生图鉴` `ba羁绊` 数据源更换为`SchaleDB`\n- 原`ba学生图鉴`修改为`ba学生wiki`\n\n### 0.3.0\n\n- 新指令 `baL2D`\n- 新指令 `ba羁绊`\n\n### 0.2.2\n\n- 添加学生别名判断\n- 修改日程表图片宽度\n\n### 0.2.1\n\n- 修改页面加载等待的事件，可能修复截图失败的问题\n\n### 0.2.0\n\n- 新指令 `ba新学生` （详情使用 [nonebot-plugin-PicMenu](https://github.com/hamo-reid/nonebot_plugin_PicMenu) 查看）\n\n### 0.1.1\n\n- 日程表改为以图片形式发送\n- 日程表不会显示未开始的活动了\n- 小 bug 修复\n- ~~移除了 herobrine~~\n',
    'author': 'student_2333',
    'author_email': 'lgc2333@126.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/lgc2333/nonebot-plugin-bawiki/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
